/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*
 * Copyright (C) 2002 CodeFactory AB
 * Copyright (C) 2002 Richard Hult <rhult@codefactory.se>
 * Copyright (C) 2002 Tim Waugh <twaugh@redhat.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <config.h>
#include <errno.h>
#include <signal.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <time.h>
#include <unistd.h>
#include <gdk/gdk.h>
#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>
#include <libgnomeui/gnome-stock-icons.h>
#include "eggtrayicon.h"
#include "lp-intl.h"
#include "lpcups.h"
#include "lpegg.h"

struct _LpEgg {
	/* Widgets. */
	EggTrayIcon   *icon;
	GtkWidget     *icon_image;
	GdkPixbuf     *jobs;

	unsigned int jobs_in_queue;

	pid_t print_manager;

	GtkItemFactory *popup_factory;
	GtkTooltips *tips;
	gchar *tip;
};

static void
popup_about_cb (gpointer   callback_data,
		guint      action,
		GtkWidget *widget)
{
	static GtkWidget *about_window;
	GtkWidget        *vbox;
	GtkWidget        *label;
	gchar            *markup;

	if (about_window) {
		gtk_window_present (GTK_WINDOW (about_window));
		return;
	}
	
	about_window = gtk_dialog_new ();

	g_object_add_weak_pointer (G_OBJECT (about_window),
				   (gpointer *) &about_window);
	
	gtk_dialog_add_button (GTK_DIALOG (about_window),
			       GTK_STOCK_OK, GTK_RESPONSE_OK);
	gtk_dialog_set_default_response (GTK_DIALOG (about_window),
					 GTK_RESPONSE_OK);

	gtk_window_set_title (GTK_WINDOW (about_window), _("About eggcups"));
	
	gtk_window_set_resizable (GTK_WINDOW (about_window), FALSE);
	gtk_window_set_position (GTK_WINDOW (about_window), 
				 GTK_WIN_POS_CENTER_ON_PARENT);
	gtk_window_set_type_hint (GTK_WINDOW (about_window), 
				  GDK_WINDOW_TYPE_HINT_DIALOG);

	vbox = gtk_vbox_new (FALSE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (vbox), 6);
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (about_window)->vbox),
			    vbox, FALSE, FALSE, 0);

	label = gtk_label_new (NULL);
	gtk_misc_set_alignment (GTK_MISC (label), 0.5, 0.5);
	gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_CENTER);
	markup = g_strdup_printf ("<span size=\"xx-large\" weight=\"bold\">%s " VERSION "</span>\n\n"
				  "%s\n\n"
				  "<span size=\"small\">%s</span>\n"
				  "<span size=\"small\">%s</span>\n",
				  _("eggcups"),
				  _("A print-queue panel icon."),
				  _("Copyright (C) 2002, Tim Waugh &lt;twaugh@redhat.com&gt;"),
				  _("Based on code by CodeFactory."));
	gtk_label_set_markup (GTK_LABEL (label), markup);
	g_free (markup);
	gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, TRUE, 0);
	
	gtk_widget_show_all (about_window);
	gtk_dialog_run (GTK_DIALOG (about_window));
	gtk_widget_destroy (about_window);
}

static void
popup_quit_cb (gpointer   callback_data,
	       guint      action,
	       GtkWidget *widget)
{
	GtkWidget *dialog;
	gint       response;

	dialog = gtk_message_dialog_new (NULL,
					 0,
					 GTK_MESSAGE_QUESTION,
					 GTK_BUTTONS_YES_NO,
					 _("Do you really want to quit?"));

	response = gtk_dialog_run (GTK_DIALOG (dialog));
	if (response == GTK_RESPONSE_YES)
		gtk_main_quit ();

	gtk_widget_destroy (dialog);
}

#define GIF_CB(x) ((GtkItemFactoryCallback)(x))

static GtkItemFactoryEntry popup_items[] = {
	{ N_("/_About"), NULL, GIF_CB (popup_about_cb), 0,
	  "<StockItem>",  GNOME_STOCK_ABOUT },
	{ "/sep1", NULL, 0, 0, "<Separator>",  NULL },
	{ N_("/_Quit"), "", GIF_CB (popup_quit_cb), 0,
	  "<StockItem>", GTK_STOCK_QUIT },
};

extern gboolean debug;

static gboolean
maybe_change_state (LpEgg *lp)
{
	unsigned int j = get_num_jobs ();
	gchar *tip;

	if ((j ? 1 : 0) != (lp->jobs_in_queue ? 1 : 0)) {
		lp->jobs_in_queue = j;
		if (j)
			gtk_widget_show (lp->icon_image);
		else
			gtk_widget_hide (lp->icon_image);
	}

	if (j == 1)
		tip = g_strdup_printf (_("Printing '%s' on %s"),
				       singlejob_jobname, singlejob_printer);
	else
		tip = g_strdup_printf (_("%d jobs printing"), j);

	if (!lp->tip || strcmp (lp->tip, tip)) {
		gtk_tooltips_set_tip (lp->tips, GTK_WIDGET (lp->icon),
				      tip, NULL);
		if (lp->tip)
			g_free (lp->tip);

		lp->tip = tip;
	} else
		g_free (tip);

	return TRUE;
}

static void
popup_menu_position_cb (GtkMenu  *menu,
			gint     *x,
			gint     *y,
			gboolean *push_in,
			gpointer  data)
{
	GtkWidget      *w = data;
	GtkRequisition  requisition;
	gint            wx, wy;

	g_return_if_fail (w != NULL);

	gtk_widget_size_request (GTK_WIDGET (menu), &requisition);

	gdk_window_get_origin (w->window, &wx, &wy);

	if (*x < wx)
		*x = wx;
	else if (*x > wx + w->allocation.width)
		*x = wx + w->allocation.width;

	if (*x + requisition.width > gdk_screen_width())
		*x = gdk_screen_width() - requisition.width;

	if (*y < wy)
		*y = wy;
	 else if (*y > wy + w->allocation.height)
		*y = wy + w->allocation.height;

	if (*y + requisition.height > gdk_screen_height())
		*y = gdk_screen_height() - requisition.height;

	*push_in = TRUE;
}

static gboolean
icon_button_press_cb (GtkWidget      *widget,
		      GdkEventButton *event,
		      LpEgg       *lp)
{
	if (event->button == 1) {
		if (lp->print_manager != -1
		    && kill (lp->print_manager, 0) && errno == ESRCH)
			lp->print_manager = -1;

		if (lp->print_manager == -1) {
			lp->print_manager = fork ();
			if (lp->print_manager == 0) {
				GList *printers = cups_printers;

				char **argv;
				guint len = g_list_length (printers);
				argv = malloc (sizeof (char *) * (2 + len));

				argv[0] = strdup ("gnome-print-manager");

				if (0 && len == 1) {
					char *p;
					p = (char *) printers->data;
					argv[1] = g_strdup_printf ("-p%s", p);
					argv[2] = NULL;
				} else {
					GList *l = printers;
					guint i = 1;
					char *p;

					while (l) {
						p = (char *) l->data;
						p = g_strdup_printf ("-o%s",p);
						argv[i++] = p;
						l = l->next;
					}
					argv[i] = NULL;
				}

				execvp (argv[0], argv);
				exit (1);
			}
		}

		return TRUE;
	}

	if (event->button == 3) {
		GtkWidget *menu;

		menu = gtk_item_factory_get_widget (lp->popup_factory, "");
		gtk_menu_popup (GTK_MENU (menu),
				NULL,
				NULL,
				popup_menu_position_cb,
				lp->icon,
				event->button,
				event->time);
	}

	return FALSE;
}

static void
icon_destroy_cb (GtkWidget *widget, gpointer user_data)
{
	gtk_main_quit ();
}

static char *
item_factory_trans_cb (const gchar *path,
		       gpointer data)
{
	return _((gchar *) path);
}

LpEgg *
lpegg_new (void)
{
	LpEgg  *lp;

        lp = g_new0 (LpEgg, 1);

	lp->jobs_in_queue = 0;
	lp->print_manager = -1;
	lp->icon = egg_tray_icon_new (_("LpEgg"));
	lp->jobs = gdk_pixbuf_new_from_file (IMAGEDIR "/icon.png", NULL);
	lp->icon_image = gtk_image_new_from_pixbuf (lp->jobs);
	gtk_widget_hide (lp->icon_image);
	gtk_widget_show_all (GTK_WIDGET (lp->icon));
	gtk_container_add (GTK_CONTAINER (lp->icon), lp->icon_image);

	lp->popup_factory = gtk_item_factory_new (GTK_TYPE_MENU,
						  "<main>",
						  NULL);
	gtk_item_factory_set_translate_func (lp->popup_factory,
					     item_factory_trans_cb,
					     NULL,
					     NULL);

	gtk_item_factory_create_items (lp->popup_factory,
				       G_N_ELEMENTS (popup_items),
				       popup_items,
				       lp);

	gtk_widget_add_events (GTK_WIDGET (lp->icon), GDK_BUTTON_PRESS_MASK);

	g_signal_connect (lp->icon,
			  "button_press_event",
			  G_CALLBACK (icon_button_press_cb),
			  lp);

	lp->tips = gtk_tooltips_new ();

	g_signal_connect (lp->icon,
			  "destroy",
			  G_CALLBACK (icon_destroy_cb),
			  lp);

	maybe_change_state (lp);

	g_timeout_add (5000,
		       (GSourceFunc) maybe_change_state,
		       lp);

	return lp;
}
       
